define(["dojo/_base/lang", "dojo/Deferred", "dojo/when", "epi"], function (lang, Deferred, when, epi) {
  var initialValues = {};
  var _form;

  const areDatesEqual = function (initialValue, date) {
      if (typeof initialValue === "string" && date instanceof Date) {
          try {
            initialValue = new Date(initialValue);
            return initialValue.getTime() === date.getTime();
          } catch(e) {
            return false;
          }
      }
      return false;
  };

  var autosave = {
    // is autosave enabled
    // for now hardcoded, but can be send from server
    enabled: true,

    isSaving: false,

    initilize: function (form) {
      _form = form;
    },

    storeInitialValue: function (fieldName, widget) {
      if (!this.enabled) {
          return;
      }
      var value = widget.get("value");
      initialValues[fieldName] = lang.clone(value);
    },

    saveProperty: function (fieldName, widget) {
      if (!_form || !this.enabled) {
          return new Deferred().resolve(false);
      }

      var initialValue = initialValues[fieldName];
      var value = widget.get("value");

      // even when form was not saved, the date picker is sending onChange event
      // and initialValue is still string
      if (areDatesEqual(initialValue, value)) {
          return new Deferred().resolve(false);
      }

      var areEqual = epi.areEqual(
        initialValue,
        value,
        epi.compareOptions.treatFalsyAsEquals | epi.compareOptions.skipEmptyProperties
      );
      if (areEqual) {
          return new Deferred().resolve(false);
      }

      initialValues[fieldName] = lang.clone(value);
      _form._model.setProperty(fieldName, value);
      
      return this._performSave();
    },

    forceSave: function () {
      const namePropery = "icontent_name";
      _form._model.setProperty(namePropery, lang.clone(initialValues[namePropery]));

      return this._performSave();
    },

    _performSave: function () {

      this.isSaving = true;
      const timestamp = (new Date()).getTime().toString();
      parent.postMessage(JSON.stringify({ type: "embedded-editor", messageType: "start-saving", timestamp: timestamp  }), "*");

      var def = new Deferred();
      when(_form._model.save(), lang.hitch(this, function () {
        this.isSaving = false;
        const timestamp = (new Date()).getTime().toString();
        parent.postMessage(JSON.stringify({ type: "embedded-editor", messageType: "saved", timestamp: timestamp  }), "*");
        def.resolve(true);
      }));

      return def;
    }
  };

  return autosave;
});
