define([
    // dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/on",
    "dojo/Deferred",
    "dojo/when",

    // dijit
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/layout/_LayoutWidget",

    "epi/dependency",

    "epi-cms/contentediting/inline-editing/BlockEditFormContainer",

    // template
    "dojo/text!./templates/customcontainer.html",

    "./project-context-resolver",
    "./url-hash-reader",
    "./update-url",
    "epi-cms-welcome-integration/autosave",
    "epi-cms/widget/MediaSelector",
    "epi-cms/contentediting/editors/ContentReferenceEditor",
    "epi-cms/contentediting/ContentActionSupport",

    // resources
    "epi/i18n!epi/nls/button",

    "xstyle/css!./templates/container.css",

    // used in the template
    "dijit/form/Button",
    "epi-cms/contentediting/NotificationBar",
    "epi-cms-welcome-integration/handle-context-commands"
], function (
    // dojo
    declare,
    lang,
    aspect,
    on,
    Deferred,
    when,

    // dijit
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _LayoutWidget,

    dependency,

    FormContainer,

    // template
    template,

    projectContextResolver,
    urlHashReader,
    updateUrl,
    autosave,
    MediaSelector,
    ContentReferenceEditor,
    ContentActionSupport,

    resources
) {
    var ValidationErrorSeverity = {
      None: 0,
      Info: 1,
      Warning: 2,
      Error: 3
    };

    var EmbeddedFormContainer = declare([FormContainer], {
        // Patch BlockEditFormContainer to show content name and display tabs instead of groups
        // - we should also display tabs instead of groups
        //var original_setMetadataAttr = FormContainer.prototype._setMetadataAttr;

        _setMetadataAttr: function (metadata) {
            var settings = (metadata.customEditorSettings || {}).inlineBlock;

            /*if (!settings) {
                console.warn("Metadata was not initialized properly");
                settings = {};
            }

            if (!settings.showNameProperty) {
                this._hideProperty(metadata, "icontent_name");
            }
            if (!settings.showCategoryProperty) {
                this._hideProperty(metadata, "icategorizable_category");
            }*/
            this._moveToAnotherGroup(metadata, "iroutable_routesegment", "Advanced", -110);
            this._moveToAnotherGroup(metadata, "PageExternalURL", "Advanced", -105);
            this._moveToAnotherGroup(metadata, "PageVisibleInMenu", "Advanced", -100);

            if (this.get("isTranslationNeeded")) {
              this._disableNonLanguageSpecificProperties(metadata);
            }

            metadata.properties.forEach(function (property) {
              property.settings.isInQuickEditMode = true; // set isInQuickEditMode to make inline create working
              property.settings.quickEditBlockId = this.contentLink;
            }.bind(this));

            var hiddenGroups = settings.hiddenGroups.map(function (x) {
              return x.toLowerCase();
            });

            //Kingfisher - show all tabs
            hiddenGroups= [];

            //hiddenGroups.push("EPiServerCMS_SettingsPanel".toLowerCase());
            /*var numberOfVisibleGroups = metadata.groups.filter(function (x) {
                return x.name !== "EPiServerCMS_SettingsPanel" && hiddenGroups.indexOf(x.name.toLowerCase()) === -1;
            }).length;*/

            // Change group properties container to epi-cms/layout/CreateContentGroup
            //metadata.layoutType = "epi/shell/layout/SimpleContainer";
            metadata.groups.forEach(function (group) {
              /*group.uiType = "epi-cms/layout/CreateContentGroupContainerNoHeader";

              if (group.name === "EPiServerCMS_SettingsPanel") {
                  group.options = {};
                  group.title = null;
              } else {
                  if (numberOfVisibleGroups > 1) {
                      group.uiType = "epi-cms/layout/CreateContentGroupContainer";
                  }
              }*/

              // by default Settings tab is not displayed
              if (hiddenGroups.indexOf(group.name.toLowerCase()) !== -1) {
                group.displayUI = false;
              }
            });
            this._set("metadata", metadata);

            this.startup();
        },

        _moveToAnotherGroup: function (metadata, propertyName, newGroupName, displayOrder) {
            metadata.properties.some(function (property) {
              if (property.name === propertyName) {
                property.groupName = newGroupName;
                property.displayOrder = displayOrder;
                return true;
              }
              return false;
            });
        }
    });

    return declare([_LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin], {
        templateString: template,
        res: resources,

        startup: function () {
            this.inherited(arguments);

            this._initializeProject().then(function () {
              this._initializeForm();
            }.bind(this));

            this._handleParentWindowMessages();
        },

        _handleParentWindowMessages: function() {
            var self = this;

            window.addEventListener("message", function (event) {
              var data;
              try {
                  data = JSON.parse(event.data);
              } catch(e) {
                  return;
              }
              if (data.type !== "getIframeStatus" || !data.timestamp) {
                  return;
              }

              //TODO: when get validation exception method is available in CMS then we can use it
              var isFormValid = self._form.isValid() && (self._form._model.validationErrors || []).length === 0;

              data = { type: "returnIframeStatus", timestamp: data.timestamp, status: {
                    isSaving: autosave.isSaving,
                    isValid: isFormValid
                  }};
              parent.postMessage(JSON.stringify(data), "*");
            }, false);
        },

        _initializeForm: function () {
            var container = document.createElement("div");
            this.formContainer.appendChild(container);

            this._form = new EmbeddedFormContainer({
                isInlineCreateEnabled: true
            }, container, "last");

          var handle = this.connect(this._form, "onGroupCreated", function(name, widget) {
              if (name === "ePiServerCMS_SettingsPanel") {
                if (widget._contentDetailsDef) {
                    widget._contentDetailsDef.then(function(contentDetailsWidget) {
                      contentDetailsWidget.domNode.classList.add("dijitHidden");
                      parent.postMessage(JSON.stringify({ type: "embedded-editor", messageType: "initialized"  }), "*");
                    });
                }
              }
          });
          this.own(handle);

          handle = this.connect(this._form, "onFieldCreated", function (fieldName, widget) {
            if (this._isMediaSelector(widget)) {
              widget.selector.contentSelectorPnP.contentSelectorDialogSettings.isInQuickEditMode = false;
            }
            
            autosave.storeInitialValue(fieldName, widget);
            this.own(aspect.after(widget, "onChange", function (value) {
              autosave.saveProperty(fieldName, widget).then(function(result) {
                if (result) {
                  this._handleSaveErrors();
                }
              }.bind(this));
              if (fieldName === "icontent_name") {
                this._contentNameValue = value;
              }
            }.bind(this), true));

          }.bind(this));

          this.own(handle);
          this.own(on(this._form, "FormCreated", function(){
              if (!autosave.enabled) {
                this.footer.classList.remove("dijitHidden");
              }
          }.bind(this)));

          this._form.set("contentLink", urlHashReader.getContentLink()).then(function (formContentData) {
              updateUrl(true);
          }.bind(this));

          autosave.initilize(this._form);

          const self = this;
          var registry = dependency.resolve("epi.storeregistry");
          var contentDataStore = registry.get("epi.cms.contentdata");
          this.own(aspect.after(contentDataStore, "executeMethod", function (deffered, functionParams) {
            
            // workaround: when quickedit a share block and publish it,
            // we need to force to save the current page in order to make backend send event to CMP
            // (for updating new version of preview url)
            if (functionParams[2] && functionParams[2].action === ContentActionSupport.saveAction.Publish) {
              when(deffered, function (response) {
                when(autosave.forceSave(), function() {
                  self._handleSaveErrors();
                });
              });
            }

            return deffered;
          }));
        },

        _initializeProject: function () {
           var def = new Deferred();

          var projectId = urlHashReader.getProjectId();
          if (!projectId) {
              console.warn("project Id was not set in URL");
              return def.resolve();
          }

          projectContextResolver(projectId).then(function() {
            def.resolve();
          });

           return def.promise;
        },

        _onSave: function () {
            if (this._destroyed) {
                return;
            }

            this.notificationBar.clear();

            if (!this._form.isValid()) {
                return;
            }

            if (this._form._model.contentData.name !== this._contentNameValue) {
              this._form._model.setProperty("icontent_name", this._contentNameValue);
            }
            this._form.saveForm().then(function () {
                if (!this._handleSaveErrors()) {
                    return;
                }
                var message = { type: "onClose", status: "save", content: this._form._model.contentData.properties };
                parent.postMessage(JSON.stringify(message), "*");
            }.bind(this));
        },

        _handleSaveErrors: function () {
            this.notificationBar.clear();
            var errors = this._form._model.validationErrors.filter(function (validationItem) {
                return validationItem.severity === ValidationErrorSeverity.Error;
            });
            if (errors && errors.length > 0) {
              errors.forEach(function (validationItem) {
                this.notificationBar.add({
                  content: validationItem.errorMessage
                });
              }, this);
              this.domNode.scrollTo({ top: 0, behavior: "smooth" });
              return false;
            }
            return true;
        },

        _onCancel: function () {
            var message = { type: "onClose", status: "cancel" };
            parent.postMessage(JSON.stringify(message), "*");
        },

        _isMediaSelector: function (widget) {

          if (widget.isInstanceOf(ContentReferenceEditor)
            && widget.selector
            && widget.selector.isInstanceOf(MediaSelector)) {
              return true;
            }
         
            return false;
        }
    });
});
