﻿define([
  "dojo",
  "dojo/_base/declare",
  "dojo/Deferred",
  "dojo/topic",

  "dijit/registry",

  "epi/_Module",
  "epi/dependency",

  "./project-context-resolver",
  "./url-hash-reader",
  "./update-url",
  "./edit-mode-detect"
], function (
  dojo,
  declare,
  Deferred,
  topic,

  registry,

  _Module,
  dependency,

  projectContextResolver,
  urlHashReader,
  updateUrl,
  editModeDetect
) {
  var numberOfAttempts = 0;

  return declare([_Module], {
    initialize: function () {
      this.inherited(arguments);
      var isInEditMode = editModeDetect();
      if (!isInEditMode) {
        // Because CMS store editlanguage in local storage and automatically load form in that language,
        // so we need to change the default editlanguage first to make the FormEditor load in correct one.
        var profile = this.profile || dependency.resolve("epi.shell.Profile");
        var viewSettingManager = dependency.resolve("epi.viewsettingsmanager");
        profile.set("editlanguage", viewSettingManager.getSettingProperty("viewlanguage"), { location: "session" });
        return;
      }

      var projectId = urlHashReader.getProjectId();
      if (projectId) {
        projectContextResolver(projectId).then(function (project) {
          console.log("project initialized from URL");
          var projectInitializePromise = new Deferred();
          this._ensureProjectInitialized(project.name, projectId, projectInitializePromise);

          projectInitializePromise.then(function () {
            this._ensureContentInitializedFromQueryString();
          }.bind(this));
        }.bind(this));
      }
    },

    _ensureContentInitializedFromQueryString: function () {
      var params = new URLSearchParams(window.location.search);
      var contentLink = params.get("context");
      if (!contentLink) {
        return;
      }
      topic.publish("/epi/shell/context/request", { uri: "epi.cms.contentdata:///" + contentLink }, { sender: this });
      updateUrl(false);
    },

    _ensureProjectInitialized: function (projectName, projectId, projectInitializePromise) {
      function trySetProject() {
        // HACK: we need to find project toolbar selector, assume that there is only one instance
        // and check if project was initialized
        var el = document.querySelector(".epi-project-mode-toolbar__selector");
        if (!el) {
          return false;
        }
        var widget = registry.byNode(el);
        if (!widget) {
          return false;
        }

        if (widget.get("label") === projectName) {
          return true;
        }

        projectContextResolver(projectId);
        return false;
      }

      setTimeout(function () {
        if (trySetProject() === false) {
          if (numberOfAttempts < 10) {
            this._ensureProjectInitialized(projectName, projectId, projectInitializePromise);
          }
        } else {
          projectInitializePromise.resolve();
          return;
        }
        numberOfAttempts++;
      }.bind(this), 200);
    }
  });
});
