# Make a site to be CMP ready

## Configure nuget feed
`
<add key="CmpPublishing-beta" value="https://pkgs.dev.azure.com/EpiserverEngineering/netCore/_packaging/Kingfisher/nuget/v3/index.json" />
`

## Install package
`dotnet add package Optimizely.Cms.Cmp.Publishing --version xxxx
`
## Add client and secret id in the appsettings.json 
```
"EPiServer": {
    "Cms": {
      "Service": {
        "OauthClients": [
          {
            "ClientId": "your client id",
            "ClientSecret": "your secret id",
            "AllowActAs": true
          }
        ]
      },
      "ContentMarketingPlatform": {
        "PreviewTokenTimeout": "00:05:00"
      }
    }
    "Cmp": {
      "Client": {
        "ClientId": "your client id",
        "ClientSecret": "your client secret"
      }
    }    
```


Add following code in ConfigureServices
 
```
    using Optimizely.Cms.Preview1;
    using Optimizely.Cms.DependencyInjection;
    using Microsoft.Extensions.DependencyInjection;
 
    private readonly IWebHostEnvironment _webHostingEnvironment;
    private readonly IConfiguration _configuration;

    public Startup(
        IWebHostEnvironment webHostingEnvironment, 
        IConfiguration configuration)
    {
        _webHostingEnvironment = webHostingEnvironment;
        _configuration = configuration;
    }

    public void ConfigureServices(IServiceCollection services)
    {
        ...
        services.AddCmsCmpPublishing();

        services.Configure<ProtectedModuleOptions>(
            pm =>
            {
                pm.RootPath = "/ui/";
            });

        services.Configure<UIOptions>(
            pm =>
            {
                pm.EditUrl = new Uri("/ui/cms/", UriKind.RelativeOrAbsolute);
            });
            
        if (_webHostingEnvironment.IsDevelopment())
        {
          // configure dev signing credentials to call Integration API
          services.Configure<CmsServiceOptions>(o =>
          {
              o.AddDevelopmentSigningCredentials();
          });
        }
    }


  
    public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
    {
      ...
      // Add cors
      app.UseCors();
      app.UseCmsCmpPublishingPreviewLinks();
      ...

    }
```

### Configure Access rights for the OpenIDConnect::ClientId
```
    1. Navigate to admin mode
    2. Select Access Rights 
    3. Select the destination Page
    4. Click add User/Group and Select Client Id
    5. Give the correct permission for the Client Id 
```
